#include "NativeModule.h"

#include <sensrvchannel.h>
#include <sensrvchannelfinder.h>
#include <sensrvdatalistener.h>
#include <sensrvilluminationsensor.h>

class AmbientModule : public NativeModule, public MSensrvDataListener
	{

private:

	enum Functions
		{
		GetFunction
		};

	CSensrvChannel *ambientChannel;
	TInt ambient;
	TRequestStatus *status;

	~AmbientModule()
		{
		delete ambientChannel;
		}

	Runtime::Value getLight()
		{
		Runtime::Value value;
		if (ambientChannel == NULL)
			{
			//alloca channelFinder e ChannelInfoList
			CSensrvChannelFinder* finder = CSensrvChannelFinder::NewLC();
			RSensrvChannelInfoList channelList;
			CleanupClosePushL(channelList);

			//Ricerca canale
			TSensrvChannelInfo info;
			info.iChannelType = KSensrvChannelTypeIdAmbientLightData;
			finder->FindChannelsL(channelList, info);

			if (channelList.Count() > 0)//trovato light sensor
				{
				info = channelList[0];
				ambientChannel = CSensrvChannel::NewL(info);
				ambientChannel->OpenChannelL();
				//metti in ascolto sul canale
				ambientChannel->StartDataListeningL(this, 1, 1, 0);
				}

			channelList.Close();
			CleanupStack::Pop(&channelList);
			CleanupStack::PopAndDestroy(finder);
			if (ambientChannel == NULL)
				User::Leave(KErrNotSupported);
			}
		value.SetNumber(ambient);
		return value;
		}

protected:
	TInt ExpectedRuntimeVersion()
		{
		return Runtime::VERSION;
		}

	void ConstructL()
		{
		runtime->AddNativeFunctionL(_L("get"), 0, 0, GetFunction);
		runtime->AddConstantL(_L("vdark"),
				TSensrvAmbientLightData::KAmbientLightVeryDark);
		runtime->AddConstantL(_L("dark"),
				TSensrvAmbientLightData::KAmbientLightDark);
		runtime->AddConstantL(_L("tlight"),
				TSensrvAmbientLightData::KAmbientLightTwilight);
		runtime->AddConstantL(_L("light"),
				TSensrvAmbientLightData::KAmbientLightLight);
		runtime->AddConstantL(_L("bright"),
				TSensrvAmbientLightData::KAmbientLightBright);
		runtime->AddConstantL(_L("sunny"),
				TSensrvAmbientLightData::KAmbientLightSunny);
		}

	Runtime::Value ExecuteL(TInt index, Runtime::Value *params,
			TInt paramCount, TRequestStatus &status)
		{

		Runtime::Value result;
		result.SetNull();
		switch (index)
			{
			case GetFunction:
				result = getLight();
				break;
			}
		return result;
		}

	void Cancel()
		{
		if (status != NULL)
			User::RequestComplete(status, KErrCancel);
		}

	// MSensrvDataListener
	void DataReceived(CSensrvChannel& aChannel, TInt count, TInt dataLost)
		{
		TSensrvChannelInfo info = aChannel.GetChannelInfo();
		if (info.iChannelType == KSensrvChannelTypeIdAmbientLightData)
			{
			TSensrvAmbientLightData ambientData;
			TPckg<TSensrvAmbientLightData> ambientPackage(ambientData);
			aChannel.GetData(ambientPackage);
			ambient = ambientData.iAmbientLight;
			
			//callback 
			if (status != NULL)
				User::RequestComplete(status, KErrNone);
			}
		}

	// MSensrvDataListener
	void DataError(CSensrvChannel& channel, TSensrvErrorSeverity error)
		{
		}

	// MSensrvDataListener
	void GetDataListenerInterfaceL(TUid interfaceUid, TAny *&interface)
		{
		}
	};

//ENTRY POINTS
EXPORT_C NativeModule* NewAmbientModuleL()
	{
	return new (ELeave) AmbientModule;
	}

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason)
	{return KErrNone;}
#endif

